module BaseballCalc

	# A player represents a row in the Master-small.csv players data.  The Importer
	# class will read the CSV vile and bulid Player objects.  It will also build
	# BattingSummary objects and set all of the @batting_summaries for each player.

	# Player objects know how to operate on their data.  Player ojbects also know
	# how merge data from multiple BattingSummaries since some players have 
	# batting data for multiple teams within the same year.  As a result, most of
	# the computational logic lives in the Player since it has to assume it's
	# data might be spread across multipe Batting Summaries.

	class Player

		attr_reader :player_id, :birth_year, :first_name, :last_name
		attr_accessor :batting_summaries

		def initialize(data)
			@player_id = data["playerID"]
			@birth_year = data["birthYear"]
			@first_name = data["nameFirst"]
			@last_name = data["nameLast"]
			@batting_summaries = []
		end

		def full_name
			return @player_id if @first_name.blank? && @last_name.blank?
			return @last_name if @first_name.blank?
			return @first_name if @last_name.blank?
			"#{@first_name} #{@last_name}"
		end

		def batting_summaries_for_year(year)
			@batting_summaries.select {|summary| summary.year_id == year}
		end

		def batting_summary_for_year_and_team(year, team)
			@batting_summaries.detect {|summary| summary.year_id == year && summary.team_id == team}
		end

		def batting_summaries_for_year_and_league(year, league)
			@batting_summaries.select {|summary| summary.year_id == year && summary.league == league}
		end

		def at_bats_at_least?(threshold, year)
			summaries = batting_summaries_for_year(year)
			total = summaries.inject(0) {|sum, summary| sum + summary.at_bats}
			total >= threshold
		end

		def batting_avg_improvement(year1, year2)
			start_summaries = batting_summaries_for_year(year1)
			return 0.0 if start_summaries.blank?
			end_summaries = batting_summaries_for_year(year2)
			return 0.0 if end_summaries.blank?
			
			total_start_at_bats = start_summaries.inject(0) {|sum, summary| sum + summary.at_bats}
			total_start_hits = start_summaries.inject(0) {|sum, summary| sum + summary.hits}
			total_end_at_bats = end_summaries.inject(0) {|sum, summary| sum + summary.at_bats}
			total_end_hits = end_summaries.inject(0) {|sum, summary| sum + summary.hits}

			return 0.0 if total_start_at_bats == 0 || total_end_at_bats == 0 

			start_batting_avg = total_start_hits * 1.0 / total_start_at_bats
			end_batting_avg = total_end_hits * 1.0 / total_end_at_bats

			change = (end_batting_avg - start_batting_avg).round(4)

			if player_id == "denorch01"
				puts "Start - #{start_batting_avg}"
				puts "End - #{end_batting_avg}"
			end
			change
		end

		def played_for?(team, year)
			!batting_summary_for_year_and_team(year,team).nil?
		end

		def played_in_league_during_year?(league, year)
			!@batting_summaries.detect {|summary| summary.year_id == year && summary.league == league}.nil?
		end

		def slugging_percent(year, team)
			summary = batting_summary_for_year_and_team(year, team)
			summary.slugging_percentage
		end

		def batting_avg_in_year_and_league(year,league)
			summaries = batting_summaries_for_year_and_league(year,league)
			total_hits = summaries.inject(0) {|sum, summary| sum + summary.hits}
			total_at_bats = summaries.inject(0) {|sum, summary| sum + summary.at_bats}
			return 0.0 if total_at_bats == 0
			(total_hits*1.0/total_at_bats).round(4)
		end

		def rbis_in_year_and_league(year, league)
			summaries = batting_summaries_for_year_and_league(year,league)
			summaries.inject(0) {|sum, summary| sum + summary.runs_batted_in}
		end

		def home_runs_in_year_and_league(year, league)
			summaries = batting_summaries_for_year_and_league(year,league)
			summaries.inject(0) {|sum, summary| sum + summary.home_runs}
		end

	end
end